using System;
using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using SimpleJSON;

public class ActionListManager : Persistence
{
    private int newActionCounter = 1;
    private JSONStorableStringChooser actionsChooser;

    private ActionTextFieldsUI actionTextFieldsUI = new ActionTextFieldsUI();

    private LinkedActionsUI linkedActionsUI = new LinkedActionsUI();

    private ActionItem deferredActionItem;

    private ActionItem lastAction;

    private List<Trigger> activeTriggers = new List<Trigger>();

    public override void Init()
    {
        try
        {
            InitStorables();
            InitOptionsUI();
            InitPluginActions();
        }
        catch (Exception e)
        {
            SuperController.LogError($"{nameof(ActionListManager)}.{nameof(Init)}: {e}");
        }
    }

    public void RefreshChangebleUI()
    {
        actionTextFieldsUI.RefreshUI();
        linkedActionsUI.RefreshUI();
    }

    private void InitOptionsUI()
    {
        Utils.OnInitUI(CreateUIElement);
        actionsChooser = new JSONStorableStringChooser("Actions", actionsDictionary.Keys.ToList(), "", "Action:");
        CreateScrollablePopup(actionsChooser, false);
        actionsChooser.setCallbackFunction += OnActionSelected;

        Utils.SetupButton(this, "Test Execute Action", OnExecuteActionButtonClicked, false).buttonColor = new Color(0.45f, 1f, 0.45f);

        Utils.SetupButton(this, "Add Action", OnAddActionButtonClicked, false);

        Utils.SetupButton(this, "Duplicate Action", OnDuplicateActionButtonClicked, false);

        Utils.SetupButton(this, "Remove Action", OnRemoveActionButtonClicked, false).buttonColor = new Color(0.903f, 0.440f, 0.440f, 1.0f);

        actionTextFieldsUI.InitUI(this, linkedActionsUI, actionsChooser, actionsDictionary);
        linkedActionsUI.InitUI(this, actionTextFieldsUI, actionsChooser, actionsDictionary);
    }

    private void OnExecuteActionButtonClicked()
    {
        OnTriggerSelectedActionChange(actionsChooser.val);
        OnTriggerStateChange();
    }

    private void OnDuplicateActionButtonClicked()
    {
        string actionToDuplicate = actionsChooser.val;
        if (string.IsNullOrEmpty(actionToDuplicate))
        {
            return;
        }

        if (actionsDictionary.ContainsKey(actionToDuplicate))
        {
            ActionItem originalAction = actionsDictionary[actionToDuplicate];
            string newName = GetUniqueDuplicateName(actionToDuplicate);

            ActionItem duplicatedAction = new ActionItem(newName, SameFormatActionName(newName), originalAction.Group, originalAction.When, originalAction.Effect, originalAction.Context, originalAction.Location,
                originalAction.Wearing, originalAction.Deferred, originalAction.Layer, new EventTrigger(originalAction.Trigger));
            duplicatedAction.LinkedActions = new List<string>(originalAction.LinkedActions);

            actionsDictionary.Add(newName, duplicatedAction);

            actionsChooser.choices = actionsDictionary.Keys.ToList();
            actionsChooser.valNoCallback = newName;
            actionsChooser.val = newName;
            linkedActionsUI.RefreshUI();
        }
        else
        {
            SuperController.LogError($"Action '{actionToDuplicate}' not found in dictionary.");
        }
    }

    private string GetUniqueDuplicateName(string baseName)
    {
        int counter = 1;
        string newName;
        do
        {
            newName = $"{baseName} Copy_{counter++}";
        } while (actionsDictionary.ContainsKey(newName));
        return newName;
    }

    private void OnRemoveActionButtonClicked()
    {
        string actionToRemove = actionsChooser.val;
        if (string.IsNullOrEmpty(actionToRemove))
        {
            return;
        }

        if (actionsDictionary.ContainsKey(actionToRemove))
        {
            foreach (var key in new List<string>(actionsDictionary.Keys))
            {
                actionsDictionary[key].LinkedActions.Remove(actionToRemove);
            }

            int indexToRemove = actionsChooser.choices.IndexOf(actionToRemove);
            int nextIndex = 0;
            if (indexToRemove >= 0 && indexToRemove < actionsChooser.choices.Count - 1)
            {
                nextIndex = indexToRemove;
            }

            actionsDictionary.Remove(actionToRemove);

            actionsChooser.choices = actionsDictionary.Keys.ToList();

            if (actionsChooser.choices.Count > 0)
            {
                actionsChooser.val = actionsChooser.choices[nextIndex];
            }
            else
            {
                actionTextFieldsUI.resetAllFields();
                linkedActionsUI.resetAllFields();
                actionsChooser.val = "";
                return;
            }

            RefreshUI();
        }
        else
        {
            SuperController.LogError("Action to remove not found in dictionary.");
        }
    }

    private void OnActionSelected(string selectedAction)
    {
        if (string.IsNullOrEmpty(selectedAction))
        {
            return;
        }

        ActionItem actionItem;
        if (actionsDictionary.TryGetValue(selectedAction, out actionItem))
        {
            actionTextFieldsUI.copyFieldValues(actionItem);
            linkedActionsUI.copyFieldValues(actionItem);
            linkedActionsUI.applySameFunctionNameToggle(actionItem);

            linkedActionsUI.sameActionNameFunction.val = actionItem.Name == ActionListManager.HumanFormatActionName(actionItem.Action) ? true : false;

            actionTextFieldsUI.RefreshUI();
            linkedActionsUI.RefreshUI();
        }
    }

    private void OnAddActionButtonClicked()
    {
        string nameValue = "New Action " + newActionCounter;
        string actionValue = "new_action_" + newActionCounter++;
        string whenValue = "<type_in_when_happening>";

        ActionItem newActionItem = new ActionItem(nameValue, actionValue, "", whenValue, "", "", "", "", false, "", new EventTrigger(this, "Trigger"));

        actionsDictionary.Add(actionValue, newActionItem);
        actionsChooser.choices = actionsDictionary.Keys.ToList();
        actionsChooser.val = actionValue;

        actionTextFieldsUI.RefreshUI();
        linkedActionsUI.RefreshUI();
    }

    private void OnLinkedActionSelected(string selectedAction)
    {
        // When choosing linked actions
    }

    private void InitPluginActions()
    {
        RegisterAction(new JSONStorableAction("TriggerActionChange", OnTriggerActionChange));
        RegisterAction(new JSONStorableAction("TriggerStateChange", OnTriggerStateChange));
    }

    private void InitStorables()
    {
        RegisterString(new JSONStorableString("TriggerSelectedActionChange", "", OnTriggerSelectedActionChange));
    }

    private void OnTriggerStateChange()
    {
        string newState = getPluginByType(containingAtom, "Voxta").GetStringChooserJSONParam("State").val;
        if ((newState == "idle" || newState == "off") && deferredActionItem != null)
        {
            ExecuteAction(deferredActionItem);
            SetChooserToNewActionWhenLinkedToOthers(deferredActionItem);
            deferredActionItem = null;
        }
    }

    private void OnTriggerSelectedActionChange(string name)
    {
        if (actionsDictionary.Count > 0)
        {
            ExecuteAction(GetItemByName(name));
        }
    }

    private void OnTriggerActionChange()
    {
        string newAction = getPluginByType(containingAtom, "Voxta").GetStringJSONParam("CurrentAction").val;
        if (!string.IsNullOrEmpty(newAction))
        {
            ActionItem actionItem = FindActionRoute(newAction);
            if (actionItem.Deferred == false)
            {
                ExecuteAction(actionItem);
                SetChooserToNewActionWhenLinkedToOthers(actionItem);
            }
            else
            {
                deferredActionItem = actionItem;
            }
        }
    }

    private ActionItem FindActionRoute(string action)
    {
        foreach (string name in lastAction.LinkedActions)
        {
            ActionItem actionItem = GetItemByName(name);
            if (actionItem.Action == action)
            {
                return actionItem;
            }
        }

        SuperController.LogError("Did not find route for action = " + action + " in lastAction with name = " + lastAction.Name);
        return null;
    }

    private void SetChooserToNewActionWhenLinkedToOthers(ActionItem actionItem)
    {
        if (actionItem.LinkedActions.Count > 0)
        {
            actionsChooser.val = actionItem.Name;
        }
    }

    public void ExecuteAction(ActionItem actionItem)
    {
        var newActionsList = GetActionAndLinkedActionsAsFormattedString(actionItem);
        if (!string.IsNullOrEmpty(newActionsList))
        {
            getPluginByType(containingAtom, "Voxta").GetStringJSONParam("ActionsList").val = newActionsList;
            lastAction = actionItem;
        }

        JSONStorableString contextParam = getPluginByType(containingAtom, "Voxta").GetStringJSONParam("Context");
        string newContext = UpdateCurrentContext(actionItem);
        contextParam.val = newContext;

        actionItem.Trigger.Trigger();
        if (actionItem.Trigger.HasActionsWithActiveTimers)
        {
            activeTriggers.Add(actionItem.Trigger);
        }
    }

    private string UpdateCurrentContext(ActionItem actionItem)
    {
        // Update context object
        currentContext.FreeText = actionItem.Context;
        if (!string.IsNullOrEmpty(actionItem.Location))
        {
            currentContext.Location = actionItem.Location;
        }
        if (!string.IsNullOrEmpty(actionItem.Wearing))
        {
            currentContext.Wearing = actionItem.Wearing;
        }

        // Build context string for Voxta
        var contextBuilder = new System.Text.StringBuilder();
        if (!string.IsNullOrEmpty(currentContext.FreeText))
        {
            contextBuilder.Append(currentContext.FreeText).Append(". ");
        }
        if (!string.IsNullOrEmpty(currentContext.Location))
        {
            contextBuilder.Append("{{char}} is in ").Append(currentContext.Location).Append(". ");
        }
        if (currentContext.Wearing == "nothing")
        {
            contextBuilder.Append("{{char}} is naked.");
        }
        if (!string.IsNullOrEmpty(currentContext.Wearing))
        {
            contextBuilder.Append("{{char}} is wearing ").Append(currentContext.Wearing).Append(".");
        }

        return contextBuilder.ToString();
    }

    private string GetActionAndLinkedActionsAsFormattedString(ActionItem actionItem)
    {
        var stringBuilder = new System.Text.StringBuilder();

        foreach (var linkedActionKey in actionItem.LinkedActions)
        {
            ActionItem linkedActionItem;
            if (actionsDictionary.TryGetValue(linkedActionKey, out linkedActionItem))
            {
                stringBuilder.AppendLine($"action: {linkedActionItem.Action}");
                stringBuilder.AppendLine($"when: {linkedActionItem.When}");
                if (linkedActionItem.Effect != null && linkedActionItem.Effect != "")
                {
                    stringBuilder.AppendLine($"effect: {linkedActionItem.Effect}");
                }
                if (!string.IsNullOrEmpty(linkedActionItem.Layer))
                {
                    stringBuilder.AppendLine($"layer: {linkedActionItem.Layer}");
                    stringBuilder.AppendLine("finalLayer: false");
                }
                stringBuilder.AppendLine();
            }
        }

        return stringBuilder.ToString().TrimEnd();
    }

    public static JSONStorable getPluginByType(Atom atom, string pluginType)
    {
        foreach (string pluginId in atom.GetStorableIDs())
        {
            if (pluginId.Contains(pluginType))
            {
                return atom.GetStorableByID(pluginId);
            }
        }

        throw new NullReferenceException("Cannot find plugin of type " + pluginType + " on atom " + atom.uid);
    }

    public void Update()
    {
        try
        {
            ActionItem currentActionItem = GetItemByName(actionsChooser.val);
            if (currentActionItem != null && currentActionItem.Trigger.HasActionsWithActiveTimers)
            {
                currentActionItem.Trigger.Update();
            }
            foreach (var trigger in new List<Trigger>(activeTriggers))
            {
                if (trigger.HasActionsWithActiveTimers)
                {
                    trigger.Update();
                }
                else
                {
                    activeTriggers.Remove(trigger);
                }
            }
        }
        catch (Exception e) { SuperController.LogError("Exception caught: " + e); }
    }

    public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
    {
        base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
        actionsChooser.choices = actionsDictionary.Keys.ToList();
        actionsChooser.valNoCallback = actionsDictionary.Keys.FirstOrDefault();
        lastAction = GetItemByName(actionsChooser.val);
        RefreshUI();
    }

    private void RefreshUI()
    {
        actionsChooser.choices = actionsDictionary.Keys.ToList();
        actionsChooser.valNoCallback = actionsDictionary.Keys.FirstOrDefault();

        actionTextFieldsUI.RefreshUI();
        linkedActionsUI.RefreshUI();
    }
    public static string SameFormatActionName(string name)
    {
        // Zamień spacje na podkreślenia i przekształć wszystko na małe litery
        return name.Replace(" ", "_").ToLower();
    }

    public static string HumanFormatActionName(string input)
    {
        if (string.IsNullOrEmpty(input))
        {
            return input;
        }

        input = input.Replace("_", " ");
        return char.ToUpper(input[0]) + input.Substring(1).ToLower();
    }

    public override bool ShouldIgnore() => false;
}
