using System.Collections.Generic;
using System.Linq;
using SimpleJSON;
using System;
using UnityEngine;

public class ActionTextFieldsUI
{
    private LinkedActionsUI linkedActionsUI;

    private JSONStorableStringChooser actionsChooser;

    private Dictionary<string, ActionItem> actionsDictionary;

    private ActionListManager script;

    private JSONStorableString nameField;
    private JSONStorableString actionField;

    private JSONStorableString groupField;
    private JSONStorableString whenField;
    private JSONStorableString effectField;
    private JSONStorableString locationField;
    private JSONStorableString wearingField;
    private JSONStorableString contextField;
    private JSONStorableBool deferred;
    private List<UIDynamic> uiElements = new List<UIDynamic>();
    private UIDynamicButton triggerButton;

    public void InitUI(ActionListManager script, LinkedActionsUI linkedActionsUI, JSONStorableStringChooser actionChooser, Dictionary<string, ActionItem> actionsDictionary)
    {
        this.linkedActionsUI = linkedActionsUI;
        this.actionsChooser = actionChooser;
        this.actionsDictionary = actionsDictionary;
        this.script = script;

        nameField = new JSONStorableString("Name", "", OnNameFieldChanged);
        actionField = new JSONStorableString("Action", "", OnActionFieldChanged);
        groupField = new JSONStorableString("Group", "", OnGroupFieldChanged);
        whenField = new JSONStorableString("When", "", OnWhenFieldChanged);
        effectField = new JSONStorableString("Effect", "", OnEffectFieldChanged);
        locationField = new JSONStorableString("Location", "", OnLocationFieldChanged);
        wearingField = new JSONStorableString("Wearing", "", OnWearingFieldChanged);
        contextField = new JSONStorableString("Context", "", OnContextFieldChanged);
        deferred = new JSONStorableBool("When Speech Finished", false);
        deferred.setCallbackFunction += OnDeferredFieldChanged;

        DrawUI();
    }

    public void RefreshUI()
    {
        Utils.RemoveUIElements(script, uiElements.Cast<object>().ToList());
        uiElements.Clear();
        DrawUI();
    }

    public void resetAllFields()
    {
        nameField.val = "";
        actionField.val = "";
        whenField.val = "";
        effectField.val = "";
        locationField.val = "";
        wearingField.val = "";
        contextField.val = "";
        deferred.val = false;
    }

    public void copyFieldValues(ActionItem actionItem)
    {
        if (actionItem != null && nameField != null)
        {
            nameField.val = actionItem.Name;
            actionField.val = actionItem.Action;
            groupField.val = actionItem.Group;
            whenField.val = actionItem.When;
            effectField.val = actionItem.Effect;
            locationField.val = actionItem.Location;
            wearingField.val = actionItem.Wearing;
            contextField.val = actionItem.Context;
            deferred.val = actionItem.Deferred;
        }
    }
    private void DrawUI()
    {
        uiElements.Add(Utils.SetupTextInput(script, "Name", nameField, false));

        if (!string.IsNullOrEmpty(actionsChooser.val)
        && (ActionListManager.SameFormatActionName(actionsDictionary[actionsChooser.val].Name) != actionsDictionary[actionsChooser.val].Action
        || !linkedActionsUI.sameActionNameFunction.val))
        {
            uiElements.Add(Utils.SetupTextInput(script, "Function", actionField, false));
        }

        uiElements.Add(Utils.SetupTextInput(script, "Group", groupField, false));
        uiElements.Add(Utils.CreateTitle(script, "When", false, 24, true));
        uiElements.Add(Utils.CreateMultilineTextInput(script, whenField, false, false, 100f));
        uiElements.Add(Utils.CreateTitle(script, "Effect", false, 24, true));
        uiElements.Add(Utils.CreateMultilineTextInput(script, effectField, false, false, 70f));
        uiElements.Add(Utils.SetupTextInput(script, "Location", locationField, false));
        uiElements.Add(Utils.SetupTextInput(script, "Wearing", wearingField, false));
        uiElements.Add(Utils.CreateTitle(script, "Context", false, 24, true));
        uiElements.Add(Utils.CreateMultilineTextInput(script, contextField, false, false, 100f));
        uiElements.Add(Utils.SetupToggle(script, deferred, false));

        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            triggerButton = Utils.SetupButton(script, "Action " + actionsDictionary[actionKey].Trigger.Name, actionsDictionary[actionKey].Trigger.OpenPanel, false);
            triggerButton.buttonColor = new Color(0.9f, 0.9f, 0.2f);
            uiElements.Add(triggerButton);
        }

        SimpleTriggerHandler.LoadAssets();

        if (!string.IsNullOrEmpty(actionsChooser.val))
        {
            var selectedAction = actionsDictionary[actionsChooser.val];
            copyFieldValues(selectedAction);
        }
        else
        {
            resetAllFields();
        }
    }

    private void OnActionFieldChanged(string newActionName)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            ActionItem actionItemToChange = actionsDictionary[actionKey];
            ActionItem existing = CheckForActionDuplicates(actionItemToChange, newActionName);
            if (existing != null)
            {
                SuperController.LogError("Function name change would cause duplicate route on action = " + existing.Name);
                actionField.val = actionsDictionary[actionKey].Action;
                return;
            }
            actionsDictionary[actionKey].Action = newActionName;
        }
        linkedActionsUI.RefreshUI();
    }

    public ActionItem CheckForActionDuplicates(ActionItem actionItemToChange, string newActionValue)
    {
        foreach (var actionPair in actionsDictionary)
        {
            var actionItem = actionPair.Value;

            if (actionItem.LinkedActions.Count > 0)
            {
                var linkedActionsValues = actionItem.LinkedActions
                    .Select(name => actionsDictionary.ContainsKey(name) ? actionsDictionary[name].Action : null)
                    .ToList();

                if (linkedActionsValues.Contains(actionItemToChange.Action))
                {
                    linkedActionsValues[linkedActionsValues.IndexOf(actionItemToChange.Action)] = newActionValue;
                }

                if (linkedActionsValues.GroupBy(x => x).Any(g => g.Count() > 1))
                {
                    return actionItem;
                }
            }
        }

        return null;
    }

    public void OnNameFieldChanged(string newName)
    {
        string oldName = actionsChooser.val;
        if (string.IsNullOrEmpty(oldName) || oldName == newName)
        {
            return;
        }

        if (actionsDictionary.ContainsKey(newName))
        {
            SuperController.LogError("Action with the new name already exists.");
            nameField.val = oldName;
            return;
        }

        ActionItem actionItem;
        if (actionsDictionary.TryGetValue(oldName, out actionItem))
        {
            actionsDictionary.Remove(oldName);

            if (ActionListManager.SameFormatActionName(actionItem.Name) == actionItem.Action)
            {
                actionItem.Action = ActionListManager.SameFormatActionName(newName);
            }
            actionItem.Name = newName;

            actionsDictionary[newName] = actionItem;

            foreach (var item in actionsDictionary.Values.ToArray())
            {
                if (item.LinkedActions.Contains(oldName))
                {
                    item.LinkedActions.Remove(oldName);
                    item.LinkedActions.Add(newName);
                }
            }

            actionsChooser.choices = actionsDictionary.Keys.ToList();
            actionsChooser.valNoCallback = newName;

            linkedActionsUI.RefreshUI();
        }
    }


    private void OnGroupFieldChanged(string newValue)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].Group = newValue;
        }
        linkedActionsUI.RefreshUI();
    }

    private void OnContextFieldChanged(string newValue)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].Context = newValue;
        }
    }
    private void OnWhenFieldChanged(string newValue)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].When = newValue;
        }
    }

    private void OnEffectFieldChanged(string newValue)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].Effect = newValue;
        }
    }

    private void OnLocationFieldChanged(string newValue)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].Location = newValue;
        }
    }

    private void OnWearingFieldChanged(string newValue)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].Wearing = newValue;
        }
    }

    private void OnDeferredFieldChanged(bool newValue)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionKey].Deferred = newValue;
        }
    }
}
