using System.Collections.Generic;
using UnityEngine;
using System.Linq;
using UnityEngine.Events;

using System.Collections;

public class LinkedActionsUI
{
    private ActionListManager script;
    private ActionTextFieldsUI actionTextFieldsUI;
    public JSONStorableBool sameActionNameFunction;
    public JSONStorableString layerField;
    private List<UIDynamicTwinButton> linkedActionButtons = new List<UIDynamicTwinButton>();
    private List<UIDynamic> elementsToClean = new List<UIDynamic>();
    private JSONStorableStringChooser actionsChooser;
    private Dictionary<string, ActionItem> actionsDictionary;
    private Dictionary<string, bool> groupExpanded = new Dictionary<string, bool>();

    private Dictionary<string, List<UIDynamic>> groupActionButtons = new Dictionary<string, List<UIDynamic>>();

    private float lastClickTime = 0f;
    private const float doubleClickThreshold = 0.2f;
    private string pendingSingleClickName;

    public void InitUI(ActionListManager script, ActionTextFieldsUI actionTextFieldsUI, JSONStorableStringChooser actionChooser, Dictionary<string, ActionItem> actionsDictionary)
    {
        this.actionsChooser = actionChooser;
        this.actionsDictionary = actionsDictionary;
        this.actionTextFieldsUI = actionTextFieldsUI;
        this.script = script;

        sameActionNameFunction = new JSONStorableBool("Same Function And Name", true);
        sameActionNameFunction.setCallbackFunction += OnSameFunctionAndName;
        layerField = new JSONStorableString("Layer", "", OnLayerChanged);
    }

    public void RefreshUI()
    {
        ClearLinkedActionButtonsUI();

        string currentSelectedAction = actionsChooser.val;
        if (string.IsNullOrEmpty(currentSelectedAction))
        {
            return;
        }

        ActionItem currentlySelectedActionItem = actionsDictionary[currentSelectedAction];
        elementsToClean.Add(Utils.SetupToggle(script, sameActionNameFunction, true));

        // Grupuj akcje według nazwy grupy.
        var groupsWithActions = actionsDictionary.Values
            .GroupBy(a => a.Group)
            .OrderBy(g => g.Key)
            .ToList();

        // If no linked actions allow layering
        if (currentlySelectedActionItem.LinkedActions.Count() == 0)
        {
            elementsToClean.Add(Utils.SetupTextInput(script, "Layer", layerField, true));
            elementsToClean.Add(Utils.CreateTitle(script, "Click assign/remove, Double click execute", true, 24, true, true));
        }
        else
        {
            elementsToClean.Add(Utils.CreateTitle(script, "Click assign/remove, Double click execute", true, 24, true, true));

            // Dla akcji powiązanych.
            elementsToClean.Add(Utils.CreateTitle(script, "--------------  Linked Actions  --------------", true, 24, true));
            foreach (var group in groupsWithActions)
            {
                string groupName = group.Key;
                RefreshGroupExpansion(groupName);
                List<ActionItem> actionsInGroup = group.ToList();

                // Create or remove group button depending on whether the group is expanded.
                bool isGroupExpanded = groupExpanded.ContainsKey(groupName) && groupExpanded[groupName];
                if (actionsInGroup.Any(a => currentlySelectedActionItem.LinkedActions.Contains(a.Name) && a.Name != currentSelectedAction))
                {
                    CreateGroupButtonAndActionButtons(groupName, true);
                }
            }
        }

        // Dla pozostałych akcji.
        elementsToClean.Add(Utils.CreateTitle(script, "-----------  Remaining Actions  -----------", true, 24, true));
        foreach (var group in groupsWithActions)
        {
            string groupName = group.Key;
            RefreshGroupExpansion(groupName);
            List<ActionItem> actionsInGroup = group.ToList();

            // Create or remove group button depending on whether the group is expanded.
            bool isGroupExpanded = groupExpanded.ContainsKey(groupName) && groupExpanded[groupName];
            if (actionsInGroup.Any(a => !currentlySelectedActionItem.LinkedActions.Contains(a.Name) && a.Name != currentSelectedAction))
            {
                CreateGroupButtonAndActionButtons(groupName, false);
            }
        }

        // Dodaj przycisk "Usuń wszystkie".
        var removeAllButton = Utils.SetupButton(script, "Remove All", OnRemoveAllButtonClicked, true);
        removeAllButton.buttonColor = new Color(0.903f, 0.440f, 0.440f, 1.0f);
        elementsToClean.Add(removeAllButton);

        if (!string.IsNullOrEmpty(actionsChooser.val))
        {
            var selectedAction = actionsDictionary[actionsChooser.val];
            copyFieldValues(selectedAction);
        }
        else
        {
            resetAllFields();
        }
    }

    private void RefreshGroupExpansion(string groupName)
    {
        if (!groupExpanded.ContainsKey(groupName))
        {
            if (groupName == "")
            {
                groupExpanded.Add(groupName, true);
            }
            else
            {
                groupExpanded.Add(groupName, false);
            }
        }
    }

    public void applySameFunctionNameToggle(ActionItem actionItem)
    {
        sameActionNameFunction.val = actionItem.Name == actionItem.Action;
    }
    private void OnSameFunctionAndName(bool value)
    {
        if (!value)
        {
            actionTextFieldsUI.OnNameFieldChanged(ActionListManager.HumanFormatActionName(actionsChooser.val));
            script.RefreshChangebleUI();
        }
        else
        {
            actionsDictionary[actionsChooser.val].Action = ActionListManager.SameFormatActionName(actionsChooser.val);
            sameActionNameFunction.val = true;
            script.RefreshChangebleUI();
        }
    }

    private void OnLayerChanged(string newValue)
    {
        string actionKey = actionsChooser.val;
        if (!string.IsNullOrEmpty(actionKey) && actionsDictionary.ContainsKey(actionKey))
        {
            actionsDictionary[actionsChooser.val].Layer = newValue;
        }
    }
    private void OnRemoveAllButtonClicked()
    {
        string currentSelectedAction = actionsChooser.val;
        if (string.IsNullOrEmpty(currentSelectedAction))
        {
            SuperController.LogError("No action is currently selected.");
            return;
        }

        var currentActionItem = actionsDictionary[currentSelectedAction];
        if (currentActionItem.LinkedActions.Any())
        {
            currentActionItem.LinkedActions.Clear();

            RefreshUI();
        }
    }

    private void CreateActionButton(ActionItem actionItem, string groupName)
    {
        var button = Utils.SetupTwinButton(script, getButtonName(actionItem),
            () => OnActionButtonClicked(actionItem.Name), "←",
            () => { actionsChooser.val = actionItem.Name; }, true, 0.9f, 0.05f);

        // If it's a group button, add the action buttons to the groupActionButtons dictionary
        if (!string.IsNullOrEmpty(groupName))
        {
            if (!groupActionButtons.ContainsKey(groupName))
            {
                groupActionButtons[groupName] = new List<UIDynamic>();
            }
            groupActionButtons[groupName].Add(button);
        }

        linkedActionButtons.Add(button);
    }

    private string getButtonName(ActionItem actionItem)
    {
        return actionItem.Name + ((string.IsNullOrEmpty(actionItem.Layer)) ? "" : " (L)");
    }

    private void ClearLinkedActionButtonsUI()
    {
        // Clear the linked action buttons as before
        Utils.RemoveUIElements(script, linkedActionButtons.Cast<object>().ToList());
        Utils.RemoveUIElements(script, elementsToClean.Cast<object>().ToList());

        // Now also clear the action buttons stored for each group
        groupActionButtons.Clear();
    }

    private void AddAllActionsFromGroup(string groupName, string currentSelectedAction)
    {
        if (!string.IsNullOrEmpty(currentSelectedAction))
        {
            ActionItem currentActionItem = actionsDictionary[currentSelectedAction];
            var actionsToAdd = actionsDictionary.Values
                .Where(a => a.Group == groupName && a.Name != currentSelectedAction)
                .Select(a => a.Name);

            foreach (var action in actionsToAdd)
            {
                if (!currentActionItem.LinkedActions.Contains(action))
                {
                    currentActionItem.LinkedActions.Add(action);
                }
            }

            RefreshUI();
        }
    }

    private void RemoveAllActionsFromGroup(string groupName)
    {
        string currentSelectedAction = actionsChooser.val;
        if (!string.IsNullOrEmpty(currentSelectedAction))
        {
            ActionItem currentActionItem = actionsDictionary[currentSelectedAction];
            var actionsToRemove = actionsDictionary.Values
                .Where(a => a.Group == groupName && a.Name != currentSelectedAction)
                .Select(a => a.Name);

            foreach (var action in actionsToRemove)
            {
                currentActionItem.LinkedActions.Remove(action);
            }

            RefreshUI();
        }
    }

    private void ToggleGroup(string groupName)
    {
        // Zmiana stanu rozwinięcia grupy
        groupExpanded[groupName] = !groupExpanded[groupName];

        RefreshUI();
    }

    private void CreateGroupButtonAndActionButtons(string groupName, bool isLinked)
    {
        string currentSelectedAction = actionsChooser.val;
        // Tworzenie przycisku grupy, który dodaje wszystkie akcje z grupy
        // lub usuwa je, a także rozwinie lub zwinie grupę akcji.
        UnityAction mainAction;
        if (isLinked)
        {
            // Jeśli akcja jest już powiązana, przycisk pozwala na usunięcie całej grupy akcji.
            mainAction = () => RemoveAllActionsFromGroup(groupName);
        }
        else
        {
            // Jeśli akcja nie jest powiązana, przycisk pozwala na dodanie całej grupy akcji.
            mainAction = () => AddAllActionsFromGroup(groupName, actionsChooser.val);
        }

        // Przycisk do rozwijania i zwijania listy akcji w grupie.
        UnityAction toggleAction = () => ToggleGroup(groupName);

        // Ustal, czy grupa jest aktualnie rozwinięta.
        bool isGroupExpanded = groupExpanded.ContainsKey(groupName) && groupExpanded[groupName];

        // Utwórz przycisk TwinButton dla grupy.
        UIDynamicTwinButton groupButton = Utils.SetupTwinButton(
            script,
            groupName != "" ? "Group: " + groupName : "Not in group", // Etykieta lewego przycisku.
            mainAction, // Akcja dla lewego przycisku.
            isGroupExpanded ? "-" : "+", // Etykieta prawego przycisku.
            toggleAction, // Akcja dla prawego przycisku.
            true, // rightSide - zakładam, że chcesz, aby przycisk był po lewej stronie.
            0.9f, // firstWidth - szerokość lewego przycisku.
            0.0f,
            true // labelBold - czy etykieta ma być pogrubiona.
        );

        linkedActionButtons.Add(groupButton);

        // Jeśli grupa jest rozwinięta, pamiętaj o dodaniu jej akcji do słownika groupActionButtons.
        if (isGroupExpanded)
        {
            foreach (var actionItem in actionsDictionary.Values.Where(a => a.Group == groupName))
                // {
                //     // Tworzenie akcji tylko wtedy, gdy grupa jest rozwinięta.
                //     CreateActionButton(actionItem.Action, isLinked, groupName);
                // }
                if ((actionItem.Name != currentSelectedAction) &&
                (isLinked && actionsDictionary[currentSelectedAction].LinkedActions.Contains(actionItem.Name) || (!isLinked && !actionsDictionary[currentSelectedAction].LinkedActions.Contains(actionItem.Name))))
                {
                    CreateActionButton(actionItem, groupName);
                }
        }

        // Przechowaj referencję do przycisku grupy, aby później móc go łatwo odnaleźć i zaktualizować.
        if (!groupActionButtons.ContainsKey(groupName))
        {
            groupActionButtons[groupName] = new List<UIDynamic>();
        }
        groupActionButtons[groupName].Add(groupButton);
    }

    private void OnActionButtonClicked(string name)
    {
        float timeSinceLastClick = Time.time - lastClickTime;
        if (timeSinceLastClick <= doubleClickThreshold)
        {
            // To jest podwójne kliknięcie
            OnActionButtonProcessDoubleClick(name);
            pendingSingleClickName = null; // Resetuj akcję oczekującą
        }
        else
        {
            // Ustawienie akcji oczekującej
            pendingSingleClickName = name;
            // Ustawienie timera do sprawdzenia czy to pojedyncze kliknięcie
            script.StartCoroutine(WaitAndProcessSingleClick(name));
        }
        lastClickTime = Time.time;
    }

    private IEnumerator WaitAndProcessSingleClick(string name)
    {
        // Poczekaj wystarczająco długo, aby upewnić się, że to nie podwójne kliknięcie
        yield return new WaitForSeconds(doubleClickThreshold);

        // Jeśli po upływie czasu, akcja oczekująca nadal jest tą samą akcją, to jest to pojedyncze kliknięcie
        if (pendingSingleClickName == name)
        {
            OnActionButtonProcessSingleClick(name);
            pendingSingleClickName = null; // Resetuj akcję oczekującą
        }
    }

    private void OnActionButtonProcessSingleClick(string name)
    {
        string currentSelectedAction = actionsChooser.val;
        if (string.IsNullOrEmpty(currentSelectedAction))
        {
            SuperController.LogError("No action is currently selected.");
            return;
        }

        ActionItem currentActionItem = actionsDictionary[currentSelectedAction];
        bool isLinked = currentActionItem.LinkedActions.Contains(name);
        if (isLinked)
        {
            currentActionItem.LinkedActions.Remove(name);
        }
        else if (!ActionItemHasSameActionName(currentActionItem, name))
        {
            if (!string.IsNullOrEmpty(currentActionItem.Layer))
            {
                layerField.val = "";
            }
            currentActionItem.LinkedActions.Add(name);
        }
        else
        {
            SuperController.LogError("Action with function name = " + actionsDictionary[name].Action + " already linked. Cannot have duplicate function names in the same route");
        }

        RefreshUI();
    }

    private void OnActionButtonProcessDoubleClick(string name)
    {
        script.ExecuteAction(actionsDictionary[name]);
    }

    private bool ActionItemHasSameActionName(ActionItem actionItem, string nameToAdd)
    {
        foreach (string name in actionItem.LinkedActions)
        {
            if (actionsDictionary[name].Action == actionsDictionary[nameToAdd].Action)
            {
                return true;
            }
        }
        return false;
    }

    public void resetAllFields()
    {
        layerField.val = "";
    }

    public void copyFieldValues(ActionItem actionItem)
    {
        if (actionItem != null && layerField != null)
        {
            layerField.val = actionItem.Layer;
        }
    }
}
